@@ -36,6 +36,7 @@ import logging
 import pickle
 import sys
 from typing import Union, Tuple, Dict, Optional
+import sqlite3
 import locale
 # Use the default locale as defined by the LANG variable
 locale.setlocale(locale.LC_ALL, '')
@@ -947,11 +948,20 @@ class RenameMoveFileWorker(DaemonProcess):
                                 self.process_rename_failure(rpd_file)
                             else:
                                 # Record file as downloaded in SQLite database
-                                self.downloaded.add_downloaded_file(
-                                    name=rpd_file.name, size=rpd_file.size,
-                                    modification_time=rpd_file.modification_time,
-                                    download_full_file_name=rpd_file.download_full_file_name
-                                )
+                                try:
+                                    self.downloaded.add_downloaded_file(
+                                        name=rpd_file.name, size=rpd_file.size,
+                                        modification_time=rpd_file.modification_time,
+                                        download_full_file_name=rpd_file.download_full_file_name
+                                    )
+                                except sqlite3.OperationalError as e:
+                                    # This should never happen because this is the only process
+                                    # writing to the database..... but just in case
+                                    logging.error(
+                                        "Database error adding download file %s: %s. Will not "
+                                        "retry.",
+                                        rpd_file.download_full_file_name, e
+                                    )
                         else:
                             move_succeeded = False
 
